@@ -6,12 +6,14 @@ import sys
 from struct import Struct
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
-from ..constants import MESSAGE_FLAG_MAP, MESSAGE_TYPE_MAP
+from ..constants import MESSAGE_FLAG_MAP, MESSAGE_TYPE_MAP, MessageFlag
 from ..errors import InvalidMessageError
 from ..message import Message
 from ..signature import SignatureType, Variant, get_signature_tree
 from .constants import BIG_ENDIAN, LITTLE_ENDIAN, PROTOCOL_VERSION
 
+MESSAGE_FLAG_INTENUM = MessageFlag
+
 MAX_UNIX_FDS = 16
 MAX_UNIX_FDS_SIZE = array.array("i").itemsize
 UNIX_FDS_CMSG_LENGTH = socket.CMSG_LEN(MAX_UNIX_FDS_SIZE)
@@ -106,18 +108,19 @@ SCM_RIGHTS = socket.SCM_RIGHTS
 EAGAIN = errno.EAGAIN
 EWOULDBLOCK = errno.EWOULDBLOCK
 
-
-HEADER_MESSAGE_ARG_NAME = {
-    1: "path",
-    2: "interface",
-    3: "member",
-    4: "error_name",
-    5: "reply_serial",
-    6: "destination",
-    7: "sender",
-    8: "signature",
-    9: "unix_fds",
-}
+HEADER_IDX_TO_ARG_NAME = [
+    "",
+    "path",
+    "interface",
+    "member",
+    "error_name",
+    "reply_serial",
+    "destination",
+    "sender",
+    "signature",
+    "unix_fds",
+]
+HEADER_UNIX_FDS_IDX = HEADER_IDX_TO_ARG_NAME.index("unix_fds")
 
 _SignatureType = SignatureType
 _int = int
@@ -256,23 +259,17 @@ class Unmarshaller:
             self._buf = bytearray()
         else:
             del self._buf[:to_clear]
-        self._message = None
-        self._pos = 0
-        self._body_len = 0
-        self._serial = 0
-        self._header_len = 0
-        self._message_type = 0
-        self._flag = 0
-        self._msg_len = 0
-        self._is_native = 0
-        self._read_complete = False
+        self._msg_len = 0  # used to check if we have ready the header
+        self._read_complete = False  # used to check if we have ready the message
         # No need to reset the unpack functions, they are set in _read_header
         # every time a new message is processed.
 
     @property
     def message(self) -> Optional[Message]:
         """Return the message that has been unmarshalled."""
-        return self._message
+        if self._read_complete:
+            return self._message
+        return None
 
     def _has_another_message_in_buffer(self) -> bool:
         """Check if there is another message in the buffer."""
@@ -440,12 +437,12 @@ class Unmarshaller:
             return Variant(SIGNATURE_TREE_N, self._read_int16_unpack(), False)
         elif signature == "ay":
             return Variant(
-                SIGNATURE_TREE_AY, self._read_array(SIGNATURE_TREE_AY_TYPES_0), False
+                SIGNATURE_TREE_AY, self.read_array(SIGNATURE_TREE_AY_TYPES_0), False
             )
         elif signature == "a{qv}":
             return Variant(
                 SIGNATURE_TREE_A_QV,
-                self._read_array(SIGNATURE_TREE_A_QV_TYPES_0),
+                self.read_array(SIGNATURE_TREE_A_QV_TYPES_0),
                 False,
             )
         elif signature == "s":
@@ -456,17 +453,17 @@ class Unmarshaller:
             return Variant(SIGNATURE_TREE_O, self._read_string_unpack(), False)
         elif signature == "as":
             return Variant(
-                SIGNATURE_TREE_AS, self._read_array(SIGNATURE_TREE_AS_TYPES_0), False
+                SIGNATURE_TREE_AS, self.read_array(SIGNATURE_TREE_AS_TYPES_0), False
             )
         elif signature == "a{sv}":
             return Variant(
                 SIGNATURE_TREE_A_SV,
-                self._read_array(SIGNATURE_TREE_A_SV_TYPES_0),
+                self.read_array(SIGNATURE_TREE_A_SV_TYPES_0),
                 False,
             )
         elif signature == "ao":
             return Variant(
-                SIGNATURE_TREE_AO, self._read_array(SIGNATURE_TREE_AO_TYPES_0), False
+                SIGNATURE_TREE_AO, self.read_array(SIGNATURE_TREE_AO_TYPES_0), False
             )
         elif signature == "u":
             return Variant(SIGNATURE_TREE_U, self._read_uint32_unpack(), False)
@@ -495,9 +492,6 @@ class Unmarshaller:
         ), self._readers[type_.children[1].token](self, type_.children[1])
 
     def read_array(self, type_: _SignatureType) -> Iterable[Any]:
-        return self._read_array(type_)
-
-    def _read_array(self, type_: _SignatureType) -> Iterable[Any]:
         self._pos += -self._pos & 3  # align 4 for the array
         self._pos += (
             -self._pos & (UINT32_SIZE - 1)
@@ -547,7 +541,7 @@ class Unmarshaller:
                 while self._pos - beginning_pos < array_length:
                     self._pos += -self._pos & 7  # align 8
                     key = self._read_string_unpack()
-                    result_dict[key] = self._read_array(child_1)
+                    result_dict[key] = self.read_array(child_1)
             else:
                 reader_1 = self._readers[child_1_token]
                 reader_0 = self._readers[child_0_token]
@@ -588,9 +582,11 @@ class Unmarshaller:
             signature_len = buf[self._pos]  # byte
             o = self._pos + 1
             self._pos += signature_len + 2  # one for the byte, one for the '\0'
+            if field_0 == HEADER_UNIX_FDS_IDX:  # defined by self._unix_fds
+                continue
             token_as_int = buf[o]
             # Now that we have the token we can read the variant value
-            key = HEADER_MESSAGE_ARG_NAME[field_0]
+            key = HEADER_IDX_TO_ARG_NAME[field_0]
             # Strings and signatures are the most common types
             # so we inline them for performance
             if token_as_int == TOKEN_O_AS_INT or token_as_int == TOKEN_S_AS_INT:
@@ -668,7 +664,6 @@ class Unmarshaller:
         self._pos = HEADER_ARRAY_OF_STRUCT_SIGNATURE_POSITION
         header_fields = self._header_fields(self._header_len)
         self._pos += -self._pos & 7  # align 8
-        header_fields.pop("unix_fds", None)  # defined by self._unix_fds
         signature = header_fields.pop("signature", "")
         if not self._body_len:
             tree = SIGNATURE_TREE_EMPTY
@@ -680,24 +675,24 @@ class Unmarshaller:
             tree = SIGNATURE_TREE_SA_SV_AS
             body = [
                 self._read_string_unpack(),
-                self._read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_1),
-                self._read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_2),
+                self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_1),
+                self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_2),
             ]
         elif signature == "oa{sa{sv}}":
             tree = SIGNATURE_TREE_OA_SA_SV
             body = [
                 self._read_string_unpack(),
-                self._read_array(SIGNATURE_TREE_OA_SA_SV_TYPES_1),
+                self.read_array(SIGNATURE_TREE_OA_SA_SV_TYPES_1),
             ]
         elif signature == "oas":
             tree = SIGNATURE_TREE_OAS
             body = [
                 self._read_string_unpack(),
-                self._read_array(SIGNATURE_TREE_OAS_TYPES_1),
+                self.read_array(SIGNATURE_TREE_OAS_TYPES_1),
             ]
         elif signature == "a{oa{sa{sv}}}":
             tree = SIGNATURE_TREE_A_OA_SA_SV
-            body = [self._read_array(SIGNATURE_TREE_A_OA_SA_SV_TYPES_0)]
+            body = [self.read_array(SIGNATURE_TREE_A_OA_SA_SV_TYPES_0)]
         elif signature == "o":
             tree = SIGNATURE_TREE_O
             body = [self._read_string_unpack()]
@@ -705,9 +700,12 @@ class Unmarshaller:
             tree = get_signature_tree(signature)
             body = [self._readers[t.token](self, t) for t in tree.types]
 
+        flags = MESSAGE_FLAG_MAP.get(self._flag)
+        if flags is None:
+            flags = MESSAGE_FLAG_INTENUM(self._flag)
         self._message = Message(
             message_type=MESSAGE_TYPE_MAP[self._message_type],
-            flags=MESSAGE_FLAG_MAP[self._flag],
+            flags=flags,
             unix_fds=self._unix_fds,
             signature=tree,
             body=body,
